statistics = {}
achievements = {}


-- Register an achievement with the system. Give a statistic_id and statistic_amount if you want automatic granting of the acihevement.
-- Valid args: 'id', 'title', 'description', 'xp', 'statistic_id', 'statistic_amount'
function registerAchievement(data)
	if ( data == nil or data.id == nil or data.title == nil ) then
		Console.warn("registerAchievement: Data requires at least an id and a title.")
		return 
	end

	if achievements[data.id] ~= nil then
		Console.warn("registerAchievement: An achievement already exists with id: " .. data.id)
		return
	end

	if ( data['xp'] == nil ) then
		data['xp'] = 500
	end

	if ( data['statistic_id'] ~= nil and data['statistic_amount'] == 0 ) then
		Console.warn("registerAchievement: If you provide statistic_id as a parameter, you must also provide statistic_amount.")
		return
	end

	data['awarded'] = nil
	achievements[data.id] = data
end


-- Register an internal statistic for the mod. 
-- Valid args: 'id', 'title', 'description', 'initial_value', 'max_value', 'hidden' 
function registerStatistic(data)
	if ( data == nil or data.id == nil or data.title == nil ) then
		Console.warn("registerStatistic: Requires at least an id and title.")
	end

	if statistics[id] ~= nil then
		Console.war("registerStatistic: A statistic already exists with id: " .. data.id)
		return
	end

	if data['initial_value'] then
		data['value'] = data['initial_value']
	else 
		data['value'] = 0
	end

	statistics[data.id] = data
end


-- Advance an internal statistic by the given amount. Unlocking achievements if you cross the threshold.
function advanceStatistic(id, amount)
	if statistics[id] then
		statistics[id].value = statistics[id].value + amount
		
		for _,achievement in pairs(achievements) do
			if achievement.statistic_id == id then
				if statistics[id].value >= achievement.statistic_amount then
					unlockAchievement(achievement.id)
				end
			end
		end

		return
	end

	Console.warn("advanceStatistic: No statistic exists with id: " .. id)
end


-- Get the current value for a given statistic.
function getStatisticValue(id)
	if statistics[id] then
		return statistics[id].value
	end

	Console.warn("getStatisticValue: No statistic exists with id: " .. id)
end


-- Unlock an achievement. Will show a popup notification eventually!
function unlockAchievement(id)
	if id == nil or achievements[id] == nil then
		Console.warn("unlockAchievement: Either you have provided a nil id or no achievement exists with that id.")
		return
	end

	if achievements[id].awarded == nil then
		achievements[id].awarded = Time.systemTime()
		hudPrint("Achievement: " .. achievements[id].title .. " unlocked.")

		if ( achievements[id].xp > 0 ) then
			for i=1,4 do
				local champion = party.party:getChampion(i)
				champion:gainExp(achievements[id].xp)
			end

			hudPrint("Gained " .. achievements[id].xp .. " XP")
		end
	end
end


-- Print debug info about statistics.
function debugStatistics()
	print("Statistics...")

	for id,stat in pairs(statistics) do
		print(stat.title .. " [" .. id .. "]: " .. stat.value)
	end
end


-- Print debug info about achievements.
function debugAchievements()
	print("Achievements...")

	for id,achievement in pairs(achievements) do
		local lockedString = "[Locked]"

		if achievement.awarded == true then
			lockedString = "[Awarded]"
		end

		print(achievement.title .. " [" .. id .. "]: " .. lockedString)
	end
end


-- registerStatistic({id="sneeze", title="Sneezes"})
-- registerAchievement({id='sneezes_10', title='10 Sneezes!', statistic_id='sneeze', statistic_amount=10})

-- debugStatistics()
-- debugAchievements()