hooks={}
debug = false

function setDebug(self,bool)
	self.debug = bool
end
-- LoG Framework 1 compatibility
function setHook(hookId,callback,priority)
	return self:set(hookId,callback,priority)
end

function set(self, hookId,callback,priority)
	local target,moduleName,method = self:parseHookId(hookId)

	if not self.hooks[target] then 
		self.hooks[target] = {}
	end
	if not self.hooks[target][method] then 
		self.hooks[target][method] = {}
	end	
	
	local overwritten = self:remove(hookId)
	priority = priority or #self.hooks[target][method]+1
	if self.debug and overwritten then
		print('Hook '..hookId..' was overwritten')
	end
	local hook = {
			_module = moduleName,
			_callback = callback,
			_id = hookId
			--_prio = priority,
		}	
	table.insert(self.hooks[target][method],priority,hook) 
	return hook
end

function setMultiple(self, hookId,callbacks,priority)
	for method,callback in pairs(callbacks) do
		self:set(hookId..'.'..method,callback)
	end
end

-- LoG Framework 1 compatibility
function removeHook(hookOrId)
	self:remove(hookOrId)
end

function remove(self,hookOrId)
	local hookId = ''
	if (type(hookOrId) == 'table') then
		hookId = hookOrId._id
	else
		hookId = hookOrId
	end
	local target,moduleName,method = self:parseHookId(hookId)
	if not self.hooks[target] or not self.hooks[target][method] then 
		return false;
	end
	local tmp = self.hooks[target][method]

	for i=1, #tmp do
		if tmp[i]['_id'] == hookId then
			table.remove(self.hooks[target][method],i) 
			if self.debug then
				print('Hook '..hookId..' was removed')
			end	
			return true
		end
	end

end

function parseHookId(self,hookId)
	local tmp = self.split(hookId,'.')
	return tmp[1],tmp[2],tmp[3]
end

function execute(self,target,method,ent,arg1,arg2,arg3,arg4,arg5,arg6)

	if not self.hooks[target] or not self.hooks[target][method] then
		return
	end
	--[[ commented out for performance optimization, uncomment for debugging
	if self.debug then 
		print('Hook',target,method,'called') 
		self:debugPrint{ent,arg1,arg2,arg3,arg4,arg5,arg6}
	end	
	]]
	local hooksTable = self.hooks[target][method]
	local retVal = nil
	for i=1, #hooksTable do
		retVal = hooksTable[i]:_callback(ent,arg1,arg2,arg3,arg4,arg5,arg6)
		-- if the current hook returns false, skip other hooks
		if retVal == false then return false end 
	end
	return retVal
end

function executeEntityHooks(self,componentName,method,ent,arg1,arg2,arg3,arg4,arg5,arg6)
	-- execute hooks by entity id first
	local finalRetVal = self:execute(ent.go.id.."@"..componentName,method,ent,arg1,arg2,arg3,arg4,arg5,arg6)
	if finalRetVal == false then return false end
	
	-- execute hooks by entity name second
	local retVal = self:execute(ent.go.name.."@"..componentName,method,ent,arg1,arg2,arg3,arg4,arg5,arg6)
	if retVal == false then return false end
	if retVal ~= nil then finalRetVal = retVal end
	
	return finalRetVal
end

function iterateHooks(self,callback,arg1,arg2,arg3)
	for target,methodTable in pairs(self.hooks) do
		for methodName,hooksTable in pairs(methodTable) do
			for _,hook in ipairs(hooksTable) do
				callback(hook,methodName,target,arg1,arg2,arg3)
			end
		end
	end
end

function removeBy(self,removeTarget,removeModule)
	self:iterateHooks(
		function(hook,methodName,target,removeTarget,removeModule)
			if (hook._module == removeModule and target == removeTarget) then
				fw.script:remove(hook)
			end
		end,
		removeTarget,
		removeModule
	)
end

-- helper methods

-- splits string by delimeter
-- Example: help.split('split.me.ok','.')
function split(p,d)
  local t, ll
  t={}
  ll=0
  if(#p == 1) then return {p} end
    while true do
      l=string.find(p,d,ll,true) -- find the next d in the string
      if l~=nil then -- if "not not" found then..
        table.insert(t, string.sub(p,ll,l-1)) -- Save it in our array.
        ll=l+1 -- save just after where we found it for searching next time.
      else
        table.insert(t, string.sub(p,ll)) -- Save what's left in our array.
        break -- Break at end, as it should be, according to the lua manual.
      end
    end
  return t
end


--For debugging
function printHooks(self)
	self:iterateHooks(function(hook,methodName,target)
		print(hook._id,methodName,target)
	end)
end

function debugPrint(self,args)
	out = {'arguments:'}
	for _,arg in ipairs(args) do
		if type(arg) == 'table' then
			if arg.go then
				table.insert(out,'{id:'..arg.go.id..',name:'..arg.go.name..'},')
			elseif arg.getName then
				table.insert(out,arg:getName()..',')
			end
		else
			table.insert(out,arg..',')
		end
	end
	print(unpack(out))
end

-- For performance validation testing
-- just run this in console and move the party
-- fw.script.performanceTest()
perfTest = {
	startTime = 0
}
function performanceTest()
	-- add 300 hooks for different targets
	for i = 1,300 do
		fw.script:set('tmp'..i..'.perftest'..i..'.onMove',function(hook) end)
	end	
	fw.script:set('party.perftest0.onMove',function()
			fw.script.perfTest.startTime = Time.systemTime()
	end)
	for i = 1,300 do
		fw.script:set('tmp2'..i..'.perftest'..i..'.onTurn',function(hook) end) -- add dummy hooks
		
		fw.script:set('party.perftest'..i..'.onMove',function(hook)
			print('hook',hook.count)
		end).count = i
	end
	fw.script:set('party.perftest21.onMove',function()
		print('Executed 302 party.onMove-hooks in: '..Time.systemTime() - fw.script.perfTest.startTime..' seconds')
	end)
end

