function fw_hook(class,hookName)
	if hook == 'all' or hookName == nil then
		local tmp = hooks[class]
		tmp.class = class
		return tmp
	end
	if not hooks[class] then
		print("FW: invalid component: "..class..'.'..hookName) 
		return 
	end
	if not hooks[class][hookName] then
		print("FW: invalid hook: "..class..'.'..hookName) 
		return 
	end	
	return hooks[class][hookName]
end

function fw_addHooks(component)
	local hooks = fw_hook(component.class)
	for  hookName,hook in pairs(hooks) do
		if component[hookName] == nil then
			component[hookName] = hook
		end
	end
	return component
end

hooks = {
	Brain = {
		onThink = function(self) 
			return fw.script:executeEntityHooks('brain','onThink',self) 
		end,
		onBloodied = function(self) 
			return fw.script:executeEntityHooks('brain','onBloodied',self) 
		end
	},
	Clickable = {
		onClick = function(self) 
			return fw.script:executeEntityHooks('clickable','onClick',self)
		end			
	},
	Animation = {
		onAnimationEvent  = function(self, event)
			return fw.script:executeEntityHooks('animation','onAnimationEvent',self)
		end
	},
	Button = {
		onActivate = function(self)
			return fw.script:executeEntityHooks('button','onActivate',self) 	
		end
	},
	Container = {
		onRemoveItem = function(self,item,slot)
			return fw.script:executeEntityHooks('container','onRemoveItem',self,item,slot) 	
		end,
		onInsertItem = function(self,item,slot)
			return fw.script:executeEntityHooks('container','onInsertItem',self,item,slot) 	
		end				
	},
	Counter = {
		onActivate = function(self)
			return fw.script:executeEntityHooks('counter','onActivate',self) 	
		end,
		onDeactivate = function(self)
			return fw.script:executeEntityHooks('counter','onDeactivate',self)
		end			
	},
	Door = {
		onOpen = function(self)
			return fw.script:executeEntityHooks('door','onOpen',self) 
		end,
		onClose = function(self)
			return fw.script:executeEntityHooks('door','onClose',self) 
		end,
		onAttackedByChampion  = function(self, champion, weapon,attack, slot)
			return fw.script:executeEntityHooks('door','onAttackedByChampion',self, champion, weapon,attack, slot)
		end
	},
	EquipmentItem = {
		onRecomputeStats  = function(self, champion) 	
			return fw.script:executeEntityHooks('equipmentitem','onRecomputeStats',self, champion) 	
		end,
		onComputeAccuracy  = function(self, champion, weapon, attack, attackType) 	
			return fw.script:executeEntityHooks('equipmentitem','onComputeAccuracy',self, champion, weapon, attack, attackType)
		end,			
		onComputeCritChance  = function(self, champion, weapon, attack, attackType)
			return fw.script:executeEntityHooks('equipmentitem','onComputeCritChance',self, champion, weapon, attack, attackType) 
		end
	},
	FloorTrigger = {
		onActivate = function(self) 	 
			return fw.script:executeEntityHooks('floortrigger','onActivate',self) 	 
		end,
		onDeactivate = function(self) 	 
			return fw.script:executeEntityHooks('floortrigger','onDeactivate',self) 	 
		end,
		onToggle = function(self) 	 
			return fw.script:executeEntityHooks('floortrigger','onToggle',self) 	 
		end,
		},
	Health = {
		onDie = function(self) 	 
			return fw.script:executeEntityHooks('health','onDie',self) 	 
		end,
	},
	ItemAction = {
		onAttack = function(self, champion, slot, chainIndex) 	 
			return fw.script:executeEntityHooks('itemaction','onAttack',self, champion, slot, chainIndex) 	 
		end,
	},
	Item = {
		onThrowAttackHitMonster = function(self, monster) 	 
			return fw.script:executeEntityHooks('item','onThrowAttackHitMonster',self, monster) 	 
		end,
		onEquipItem = function(self, champion, slot) 	 
			return fw.script:executeEntityHooks('item','onEquipItem',self, champion, slot) 	 
		end,
		onUnequipItem = function(self, champion, slot) 	 
			return fw.script:executeEntityHooks('item','onUnequipItem',self, champion, slot) 	 
		end,
	},
	Lever = {
		onActivate = function(self) 	 
			return fw.script:executeEntityHooks('lever','onActivate',self) 	 
		end,
		onDeactivate = function(self) 	 
			return fw.script:executeEntityHooks('lever','onDeactivate',self) 	 
		end,
		onToggle = function(self) 	 
			return fw.script:executeEntityHooks('lever','onToggle',self) 	 
		end,
	},
	Light = {
		onUpdate = function(self) 	 
			return fw.script:executeEntityHooks('light','onUpdate',self) 	 
		end,
	},
	Lock = {
		onActivate = function(self) 	 
			return fw.script:executeEntityHooks('lock','onActivate',self) 	 
		end,
	},
	MeleeAttack = {
		onPostAttack = function(self, champion, slot) 	 
			return fw.script:executeEntityHooks('meleeattack','onPostAttack',self, champion, slot) 	 
		end,
		onHitMonster = function(self, monster, tside, damage, champion) 	 
			return fw.script:executeEntityHooks('meleeattack','onHitMonster',self, monster, tside, damage, champion) 	 
		end,
	},
	MonsterAction = {
		onBeginAction = function(self) 	 
			return fw.script:executeEntityHooks('monsteraction','onBeginAction',self) 	 
		end,
		onEndAction = function(self) 	 
			return fw.script:executeEntityHooks('monsteraction','onEndAction',self) 	 
		end,
		onAnimationEvent = function(self, event) 	 
			return fw.script:executeEntityHooks('monsteraction','onAnimationEvent',self, event) 	 
		end,
	},
	MonsterAttack = {
		onAttack = function(self) 	 
			return fw.script:executeEntityHooks('monsterattack','onAttack',self) 	 
		end,
		onAttackHit = function(self, champion) 	 
			return fw.script:executeEntityHooks('monsterattack','onAttackHit',self, champion) 	 
		end,
		onDealDamage = function(self, champion, damage) 	 
			return fw.script:executeEntityHooks('monsterattack','onDealDamage',self, champion, damage) 	 
		end,
	},
	Monster = {
		onProjectileHit = function(self, item, damage, damageType) 	 
			return fw.script:executeEntityHooks('monster','onProjectileHit',self, item, damage, damageType) 	 
		end,
		onPerformAction = function(self, name) 	 
			return fw.script:executeEntityHooks('monster','onPerformAction',self, name) 	 
		end,
		onDamage = function(self, damage, damageType) 	 
			return fw.script:executeEntityHooks('monster','onDamage',self, damage, damageType) 	 
		end,
		onDie = function(self) 	 
			return fw.script:executeEntityHooks('monster','onDie',self) 	 
		end,
	},
	MonsterMove = {
		onUnlinkMonsterGroup = function(self) 	 
			return fw.script:executeEntityHooks('monstermove','onUnlinkMonsterGroup',self) 	 
		end,
	},
	Party = {
		onCastSpell = function(self, champion, spell) 	 
			return fw.script:execute('party','onCastSpell',self, champion, spell) 	 
		end,
		onDamage = function(self, champion, damage, damageType) 	 
			return fw.script:execute('party','onDamage',self, champion, damage, damageType) 	 
		end,
		onDie = function(self, champion) 	 
			return fw.script:execute('party','onDie',self, champion) 	 
		end,
		onAttack = function(self, champion, action, slot) 	 
			return fw.script:execute('party','onAttack',self, champion, action, slot) 	 
		end,
		onLevelUp = function(self, champion) 	 
			return fw.script:execute('party','onLevelUp',self, champion) 	 
		end,
		onReceiveCondition = function(self, champion, condition) 	 
			return fw.script:execute('party','onReceiveCondition',self, champion, condition) 	 
		end,
		onPickUpItem = function(self, item) 	 
			if fw.script:executeEntityHooks('item','onPickUpItem',item) == false then
				return false
			end
			return fw.script:execute('party','onPickUpItem',self, item) 	 
		end,
		onProjectileHit = function(self, champion, item, damage, damageType) 	 	
			return fw.script:execute('party','onProjectileHit',self, champion, item, damage, damageType) 	 
		end,
		onRest = function(self) 	 
			return fw.script:execute('party','onRest',self) 	 
		end,
		onWakeUp = function(self) 	 
			return fw.script:execute('party','onWakeUp',self) 	 
		end,
		onTurn = function(self, direction) 	 
			return fw.script:execute('party','onTurn',self, direction) 	 
		end,
		onMove = function(self, direction) 	 
			return fw.script:execute('party','onMove',self, direction) 	 
		end,
	},
	Gui = {
		onDrawGui = function(self, context) 	 
			return fw.script:execute('party','onDrawGui',self, context) 	 
		end,
		onDrawInventory = function(self, context, champion) 	 
			return fw.script:execute('party','onDrawInventory',self, context, champion) 	 
		end,
		onDrawStats = function(self, context, champion) 	 
			return fw.script:execute('party','onDrawStats',self, context, champion) 	 
		end,
		onDrawSkills = function(self, context, champion) 	 
			return fw.script:execute('party','onDrawSkills',self, context, champion) 	 
		end,
		onDrawTraits = function(self, context, champion) 	 
			return fw.script:execute('party','onDrawTraits',self, context, champion) 	 
		end,
	},
	Portal = {
		onArrival = function(self) 	 
			return fw.script:executeEntityHooks('portal','onArrival',self) 	 
		end,
	},
	Projectile = {
		onProjectileHit = function(self, what, entity) 	 
			return fw.script:executeEntityHooks('projectile','onProjectileHit',self, what, entity) 	 
		end,
	},
	ProjectileImpact = {
		onHitEntity = function(self, entity) 	 
			return fw.script:executeEntityHooks('projectileimpact','onHitEntity',self, entity) 	 
		end,
		onHitMonster = function(self, monster) 	 
			return fw.script:executeEntityHooks('projectileimpact','onHitMonster',self, monster) 	 
		end,
		onHitParty = function(self, party) 	 
			return fw.script:executeEntityHooks('projectileimpact','onHitParty',self, party) 	 
		end,
		},
	RangedAttack = {
		onPostAttack = function(self, champion, slot) 	 
			return fw.script:executeEntityHooks('rangedattack','onPostAttack',self, champion, slot) 	 
		end,
	},
	Socket = {
		onInsertItem = function(self, item) 	 
			return fw.script:executeEntityHooks('socket','onInsertItem',self, item) 	 
		end,
		onRemoveItem = function(self, item) 	 
			return fw.script:executeEntityHooks('socket','onRemoveItem',self, item) 	 
		end,
		onAcceptItem = function(self, item) 	 
			return fw.script:executeEntityHooks('socket','onAcceptItem',self, item) 	 
		end,
		},
	Spawner = {
		onActivate = function(self) 	 
			return fw.script:executeEntityHooks('spawner','onActivate',self) 	 
		end,
		},
	Surface = {
		onInsertItem = function(self, item) 	 
			return fw.script:executeEntityHooks('surface','onInsertItem',self, item) 	 
		end,
		onRemoveItem = function(self, item) 	 
			return fw.script:executeEntityHooks('surface','onRemoveItem',self, item) 	 
		end,
		},
	ThrowAttack = {
		onPostAttack = function(self, champion, slot) 	 
			return fw.script:executeEntityHooks('throwattack','onPostAttack',self, champion, slot) 	 
		end,
	},
	TileDamager = {
		onHitChampion = function(self, champion) 	 
			return fw.script:executeEntityHooks('tiledamager','onHitChampion',self, champion) 	 
		end,
		onHitMonster = function(self, monster) 	 
			return fw.script:executeEntityHooks('tiledamager','onHitMonster',self, monster) 	 
		end,
		onHitObstacle = function(self, obstacle) 	 
			return fw.script:executeEntityHooks('tiledamager','onHitObstacle',self, obstacle) 	 
		end,
	},
	Timer = {
		onActivate = function(self) 	 
			return fw.script:executeEntityHooks('timer','onActivate',self) 	 
		end,
	},
	TricksterBrain = {
		onThinkSpecial = function(self) 	 
			return fw.script:executeEntityHooks('tricksterbrain','onThinkSpecial',self) 	 
		end,
	},
	UsableItem = {
		onUseItem = function(self, champion) 	 
			return fw.script:executeEntityHooks('usableitem','onUseItem',self, champion) 	 
		end,
	},
	WallText = {
		onShowText = function(self) 	 
			return fw.script:executeEntityHooks('walltext','onShowText',self) 	 
		end,
		onDismissText = function(self) 	 
			return fw.script:executeEntityHooks('walltext','onDismissText',self) 	 
		end,
	},
	WallTrigger = {
		onActivate = function(self) 	 
			return fw.script:executeEntityHooks('walltrigger','onActivate',self) 	 
		end,
	}
}

function fw_userData()
	return {
		class = "Script",
		name = "data",
		source = [[
	data = {}
	function get(self,name)
		return self.data[name]
	end
	function set(self,name,value)
		self.data[name] = value
	end
	]],		
	}
end

userData = {
		class = "Script",
		name = "data",
		source = [[
	data = {}
	function get(self,name)
		return self.data[name]
	end
	function set(self,name,value)
		self.data[name] = value
	end
	]],		
	}

