defineObject{
	name = "xan_magma_golem",
	baseObject = "base_monster",
	components = {
		{
			class = "Model",
			model = "assets/models/monsters/magma_golem.fbx",
			storeSourceData = true, -- must be enabled for mesh particles to work
		},
		{
			class = "Animation",
			animations = {
				idle = "assets/animations/monsters/magma_golem/magma_golem_idle.fbx",
				moveForward = "assets/animations/monsters/magma_golem/magma_golem_walk.fbx",
				turnLeft = "assets/animations/monsters/magma_golem/magma_golem_turn_left.fbx",
				turnRight = "assets/animations/monsters/magma_golem/magma_golem_turn_right.fbx",
				turnAround = "assets/animations/monsters/magma_golem/magma_golem_turn_around.fbx",
				attack = "assets/animations/monsters/magma_golem/magma_golem_attack.fbx",
				attack2 = "assets/animations/monsters/magma_golem/magma_golem_punch_attack.fbx",
				groundPound = "assets/animations/monsters/magma_golem/magma_golem_ground_pound.fbx",
				rangedAttack = "assets/animations/monsters/magma_golem/magma_golem_ranged_attack.fbx",
				turnAttackLeft = "assets/animations/monsters/magma_golem/magma_golem_turn_attack_left.fbx",
				turnAttackRight = "assets/animations/monsters/magma_golem/magma_golem_turn_attack_right.fbx",
				getHitFrontLeft = "assets/animations/monsters/magma_golem/magma_golem_get_hit_front_left.fbx",
				getHitFrontRight = "assets/animations/monsters/magma_golem/magma_golem_get_hit_front_right.fbx",
				getHitBack = "assets/animations/monsters/magma_golem/magma_golem_get_hit_back.fbx",
				getHitLeft = "assets/animations/monsters/magma_golem/magma_golem_get_hit_left.fbx",
				getHitRight = "assets/animations/monsters/magma_golem/magma_golem_get_hit_right.fbx",
				fall = "assets/animations/monsters/magma_golem/magma_golem_get_hit_front_left.fbx",
				meteorFall = "assets/animations/monsters/magma_golem/magma_golem_meteor.fbx",
			},
			currentLevelOnly = true,
			onAnimationEvent = function(self, event)
				if event == "footstep" then
					if self.go.brain.partyOnLevel then
						local dist = math.abs(party.x - self.go.x) + math.abs(party.y - self.go.y)
						if dist < 5 then 
							party.party:shakeCamera(0.7 / dist, 0.3)
						end
					end
				end
			end,
		},
		{
			class = "Monster",
			meshName = "stone_elemental_mesh",
			hitSound = "magma_golem_hit",
			dieSound = "magma_golem_die",
			footstepSound = "magma_golem_footstep",
			hitEffect = "hit_flame",
			capsuleHeight = 0.2,
			capsuleRadius = 0.7,
			collisionRadius = 1.5,
			collisionHeight = 2.5,
			health = 5000,
			protection = 15,
			evasion = 0,
			exp = 0,
			immunities = { "sleep", "blinded", "knockback", "frozen", "poisoned" },
			resistances = {
				fire = "immune",
				shock = "immune",
				frost = "immune",
				poison = "immune",
			},
			traits = { "elemental" },
			
			onDie = function() return xan_ending.script.golemCanDie() end,
			onDamage = function() return false end,
			
		},
		{
			class = "MagmaGolemBrain",
			name = "brain",
			sight = 30,
			morale = 100,
		},
		{
			class = "MonsterMove",
			name = "move",
			sound = "magma_golem_walk",
			cooldown = 3,
		},
		{
			class = "MonsterTurn",
			name = "turn",
			sound = "magma_golem_walk",
		},
		{
			class = "MonsterAttack",
			name = "basicAttack",
			animation = "attack",
			attackPower = 200,
			woundChance = 40,
			cooldown = 2,
			sound = "magma_golem_attack",
			onBeginAction = function(self)
				-- randomize animation
				if math.random() < 0.5 then
					self:setAnimation("attack")
				else
					self:setAnimation("attack2")
				end
			end,
		},
		{
			class = "MonsterAttack",
			name = "turnAttack",
			attackPower = 200,
			cooldown = 3,
			sound = "magma_golem_turn_attack",
			accuracy = 70,
			woundChance = 40,
			turnToAttackDirection = true,
			cameraShake = true,
			cameraShakeIntensity = 0.7,
			screenEffect = "damage_screen",
		},
		{
			class = "MonsterAttack",
			name = "groundPound",
			animation = "groundPound",
			cooldown = 10,
			repeatChance = 0,
			sound = "magma_golem_ground_pound",
			onAttack = function(self)
				local x,y = self.go.x,self.go.y
				local dx,dy = getForward(self.go.facing)
				x = x + dx
				y = y + dy
				spawn("flame_wave", self.go.level, x, y, 0, self.go.elevation)
				party.party:shakeCamera(0.7, 0.3)
			end,
		},
		{
			class = "MonsterAttack",
			name = "rangedAttack",
			attackType = "projectile",
			cooldown = 5,
			animation = "rangedAttack",
			sound = "magma_golem_ranged_attack",
			onAttack = function(self)
				local projectile = self.go.monster:shootProjectile("magma_meteor", 3, 10, "jaw").projectile
				projectile:setVelocity(15)
				projectile:setGravity(16)
				projectile:setIgnoreEntity(self.go, 1)
				local dist = self.go.brain.partyDistY
				if dist < 3 then
					projectile:setFallingVelocity(0.1)
					projectile:setVelocity(10)
				elseif dist == 3 then
					projectile:setFallingVelocity(1)
				elseif dist == 4 then
					projectile:setFallingVelocity(2.7)
				elseif dist == 5 then
					projectile:setFallingVelocity(5.5)
				else
					projectile:setFallingVelocity(7.5)
				end

				return false
			end,
		},
		{
			class = "UggardianFlames",
			name = "aliveParticles",
			particleSystem = "magma_golem_meteor_fall",
			enabled = false,
		},
		{
			class = "UggardianFlames",
			name = "aliveAgainParticles",
			particleSystem = "magma_golem_meteor_fall",
			enabled = false,
		},
		{
			class = "Light",
			parentNode = "capsule",
			offset = vec(0, -0.6, 0.85),
			range = 5,
			color = vec(1, 0.5, 0.2),
			brightness = 4,
			fillLight = true,
			--debugDraw = true,
		},
		{
			class = "Obstacle",
			blockMonsters = false,
			blockParty = true,
			blockItems = false,
		},
	},
}
defineObject{
	name = "xan_icy_golem",
	baseObject = "base_monster",
	components = {
		{
			class = "Model",
			model = "assets/models/monsters/magma_golem.fbx",
			storeSourceData = true, -- must be enabled for mesh particles to work
			material = "xan_mtl_icy_golem",
		},
		{
			class = "Animation",
			animations = {
				idle = "assets/animations/monsters/magma_golem/magma_golem_idle.fbx",
				moveForward = "assets/animations/monsters/magma_golem/magma_golem_walk.fbx",
				turnLeft = "assets/animations/monsters/magma_golem/magma_golem_turn_left.fbx",
				turnRight = "assets/animations/monsters/magma_golem/magma_golem_turn_right.fbx",
				turnAround = "assets/animations/monsters/magma_golem/magma_golem_turn_around.fbx",
				attack = "assets/animations/monsters/magma_golem/magma_golem_attack.fbx",
				attack2 = "assets/animations/monsters/magma_golem/magma_golem_punch_attack.fbx",
				groundPound = "assets/animations/monsters/magma_golem/magma_golem_ground_pound.fbx",
				rangedAttack = "assets/animations/monsters/magma_golem/magma_golem_ranged_attack.fbx",
				turnAttackLeft = "assets/animations/monsters/magma_golem/magma_golem_turn_attack_left.fbx",
				turnAttackRight = "assets/animations/monsters/magma_golem/magma_golem_turn_attack_right.fbx",
				getHitFrontLeft = "assets/animations/monsters/magma_golem/magma_golem_get_hit_front_left.fbx",
				getHitFrontRight = "assets/animations/monsters/magma_golem/magma_golem_get_hit_front_right.fbx",
				getHitBack = "assets/animations/monsters/magma_golem/magma_golem_get_hit_back.fbx",
				getHitLeft = "assets/animations/monsters/magma_golem/magma_golem_get_hit_left.fbx",
				getHitRight = "assets/animations/monsters/magma_golem/magma_golem_get_hit_right.fbx",
				fall = "assets/animations/monsters/magma_golem/magma_golem_get_hit_front_left.fbx",
				meteorFall = "assets/animations/monsters/magma_golem/magma_golem_meteor.fbx",
			},
			currentLevelOnly = true,
			onAnimationEvent = function(self, event)
				if event == "footstep" then
					if self.go.brain.partyOnLevel then
						local dist = math.abs(party.x - self.go.x) + math.abs(party.y - self.go.y)
						if dist < 5 then 
							party.party:shakeCamera(0.7 / dist, 0.3)
						end
					end
				end
			end,
		},
		{
			class = "Monster",
			meshName = "stone_elemental_mesh",
			hitSound = "magma_golem_hit",
			dieSound = "magma_golem_die",
			footstepSound = "magma_golem_footstep",
			hitEffect = "hit_ice",
			deathEffect = "death_icy",
			capsuleHeight = 0.2,
			capsuleRadius = 0.7,
			collisionRadius = 1.5,
			collisionHeight = 2.5,
			health = 5000,
			protection = 15,
			evasion = 0,
			exp = 0,
			immunities = { "sleep", "blinded", "knockback", "frozen", "poisoned" },
			resistances = {
				fire = "immune",
				shock = "immune",
				frost = "immune",
				poison = "immune",
			},
			traits = { "elemental" },
			
			onDie = function() return xan_ending.script.golemCanDie() end,
			onDamage = function() return false end,
		},
		{
			class = "MagmaGolemBrain",
			name = "brain",
			sight = 30,
			morale = 100,
		},
		{
			class = "MonsterMove",
			name = "move",
			sound = "magma_golem_walk",
			cooldown = 3,
		},
		{
			class = "MonsterTurn",
			name = "turn",
			sound = "magma_golem_walk",
		},
		{
			class = "MonsterAttack",
			name = "basicAttack",
			animation = "attack",
			attackPower = 300,
			woundChance = 40,
			cooldown = 2,
			sound = "magma_golem_attack",
			onBeginAction = function(self)
				-- randomize animation
				if math.random() < 0.5 then
					self:setAnimation("attack")
				else
					self:setAnimation("attack2")
				end
			end,
		},
		{
			class = "MonsterAttack",
			name = "turnAttack",
			attackPower = 300,
			cooldown = 3,
			sound = "magma_golem_turn_attack",
			accuracy = 70,
			woundChance = 40,
			turnToAttackDirection = true,
			cameraShake = true,
			cameraShakeIntensity = 0.7,
			screenEffect = "damage_screen",
		},
		{
			class = "MonsterAttack",
			name = "groundPound",
			animation = "groundPound",
			cooldown = 1,
			repeatChance = 50,
			sound = "magma_golem_ground_pound",
			onAttack = function(self)
				if (xan_magma_golem_2) then
					for x = -1, 1 do
						for y = -1, 1 do
							local spell = spawn("ice_shards", xan_magma_golem_2.level, xan_magma_golem_2.x + x, xan_magma_golem_2.y + y, 0, -1)
							spell.iceshards:setRange(1)
							spell.tiledamager:setAttackPower(132)
						end
					end
				end
			end,
		},
		{
			class = "MonsterAttack",
			name = "rangedAttack",
			attackType = "projectile",
			cooldown = 5,
			animation = "rangedAttack",
			sound = "magma_golem_ranged_attack",
			onAttack = function(self)
				local projectile = self.go.monster:shootProjectile("icy_meteor", 3, 10, "jaw").projectile
				projectile:setVelocity(15)
				projectile:setGravity(16)
				projectile:setIgnoreEntity(self.go, 1)
				local dist = self.go.brain.partyDistY
				if dist < 3 then
					projectile:setFallingVelocity(0.1)
					projectile:setVelocity(10)
				elseif dist == 3 then
					projectile:setFallingVelocity(1)
				elseif dist == 4 then
					projectile:setFallingVelocity(2.7)
				elseif dist == 5 then
					projectile:setFallingVelocity(5.5)
				else
					projectile:setFallingVelocity(7.5)
				end

				return false
			end,
		},
		{
			class = "UggardianFlames",
			name = "aliveParticles",
			particleSystem = "ice_golem_particles",
			enabled = false,
		},
		{
			class = "UggardianFlames",
			name = "aliveAgainParticles",
			particleSystem = "ice_golem_particles",
			enabled = false,
		},
		{
			class = "Light",
			parentNode = "capsule",
			offset = vec(0, -0.6, 0.85),
			range = 5,
			color = vec(1, 0.5, 0.2),
			brightness = 4,
			fillLight = true,
			--debugDraw = true,
		},
		{
			class = "Obstacle",
			blockMonsters = false,
			blockParty = true,
			blockItems = false,
		},		
	},
}

defineParticleSystem{
	name = "ice_golem_particles",
	emitters = {
		-- smoke
		{
			emissionRate = 150,
			emitterShape = "MeshShape",
			emissionTime = 0,
			maxParticles = 400,
			boxMin = {-0.15, -1,-0.15},
			boxMax = { 0.15, 0.75, 0.15},
			sprayAngle = {150,180},
			velocity = {0.5,0},
			texture = "assets/textures/particles/ice_guardian_smoke.tga",
			lifetime = {0.5,2.0},
			color0 = {0.5,0.5,0.5},
			opacity = 1,
			fadeIn = 0.5,
			fadeOut = 0.5,
			size = {0.5,1},
			gravity = {0,-0.1,0},
			airResistance = 0.5,
			rotationSpeed = 0.5,
			--blendMode = "Translucent",
			blendMode = "Additive",
			objectSpace = true,
		},
		-- smoke
		{
			emissionRate = 40,
			emissionTime = 0,
			maxParticles = 40,
			boxMin = {-0.15, -1,-0.15},
			boxMax = { 0.15, 0.75, 0.15},
			sprayAngle = {150,180},
			velocity = {0.5,0},
			texture = "assets/textures/particles/ice_guardian_smoke.tga",
			lifetime = {0.5,1.1},
			color0 = {0.5,0.5,0.5},
			opacity = 0.5,
			fadeIn = 0.5,
			fadeOut = 1,
			size = {0.5,1},
			gravity = {0,-0.1,0},
			airResistance = 0.5,
			rotationSpeed = 0.5,
			--blendMode = "Translucent",
			blendMode = "Additive",
		},
		--sparks
		{
			emissionRate = 80,
			emissionTime = 0,
			maxParticles = 50,
			boxMin = {-0.25, -1,-0.25},
			boxMax = { 0.25, 1, 0.25},
			sprayAngle = {150,180},
			velocity = {0.1,0.6},
			objectSpace = true,
			texture = "assets/textures/particles/glitter_silver.tga",
			lifetime = {0.5,1},
			color0 = {5,5,5},
			opacity = 0.5,
			fadeIn = 0.1,
			fadeOut = 0.1,
			size = {0.04, 0.08},
			gravity = {0,0,0},
			airResistance = 0.04,
			rotationSpeed = 10,
			blendMode = "Additive",
		},
	}

}

defineObject{
	name = "icy_meteor",
	baseObject = "base_spell",
	components = {
		{
			class = "Particle",
			particleSystem = "frostbolt",
		},
		{
			class = "Light",
			color = vec(1, 0.5, 0.25),
			brightness = 15,
			range = 7,
			castShadow = true,
			fillLight = true,
		},
		{
			class = "Sound",
			sound = "frostbolt",
		},
		{
			class = "Sound",
			name = "launchSound",
			sound = "frostbolt_launch",
		},
		{
			class = "Projectile",
			spawnOffsetY = 1.35,
			velocity = 10,
			radius = 0.1,
			--hitEffect = "hit_ice",
			onProjectileHit = function(self, what, entity)
				if (xan_magma_golem_2) then
					for x = -1, 1 do
						for y = -3, 1 do
							local spell = spawn("ice_shards", xan_magma_golem_2.level, xan_magma_golem_2.x + x, xan_magma_golem_2.y + y, 0, -1)
							spell.iceshards:setRange(1)
							spell.tiledamager:setAttackPower(132)
						end
					end
				end
			end,
		},
	},
}

-- xan_mtl_icy_golem

defineMaterial{
	name = "xan_mtl_icy_golem",
	diffuseMap = "assets/textures/monsters/frozen_dif.tga",
	doubleSided = false,
	lighting = true,
	ambientOcclusion = false,
	alphaTest = false,
	blendMode = "Opaque",
	textureAddressMode = "Wrap",
	glossiness = 20,
	depthBias = 0,
		
	-- custom shader
	shader = "crystal",
	shadeTex = "assets/textures/env/healing_crystal_shadetex.tga",
	shadeTexAngle = 0,
	crystalIntensity = 3,
	onUpdate = function(self, time)
		self:setParam("shadeTexAngle", time*1.3)
	end,
}

