defineObject{
	name = "xan_garth_guardian",
	baseObject = "base_monster",
	components = {
		{
			class = "Model",
			model = "assets/models/monsters/skeleton_knight_commander.fbx",
			storeSourceData = true, -- must be enabled for mesh particles to work
		},
		{
			class = "Animation",
			animations = {
				idle = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_idle.fbx",
				moveForward = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_walk.fbx",
				turnLeft = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_turn_left.fbx",
				turnRight = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_turn_right.fbx",
				attack = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_attack.fbx",
				attack2 = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_attack2.fbx",
				moveAttack = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_move_attack.fbx",
				rangedAttack = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_ranged_attack.fbx",
				summonUndead = "mod_assets/rooms/xan/assets/guardian_cast_spell.fbx",
				turnAttackLeft = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_turn_attack_left.fbx",
				turnAttackRight = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_turn_attack_right.fbx",
				turnAroundAttack = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_turn_around_attack.fbx",
				getHitFrontLeft = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_get_hit_front_left.fbx",
				getHitFrontRight = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_get_hit_front_right.fbx",
				getHitBack = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_get_hit_back.fbx",
				getHitLeft = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_get_hit_left.fbx",
				getHitRight = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_get_hit_right.fbx",
				fall = "assets/animations/monsters/skeleton_knight_commander/skeleton_knight_commander_get_hit_front_left.fbx",
			},
			currentLevelOnly = true,
			onAnimationEvent = function(self, event)
				if event == "summon_undead_begin" then
					self.go.handLeftParticle:restart()
					self.go.handRightParticle:restart()
					self.go.spellLight:fadeIn(2.7)
				elseif event == "summon_undead_end" then
					self.go.spellLight:fadeOut(0.1)
				end
			end,
		},
		{
			class = "Monster",
			meshName = "skeleton_knight_commander_mesh",
			footstepSound = "skeleton_footstep",
			hitSound = "skeleton_hit",
			dieSound = "skeleton_die",
			hitEffect = "hit_dust",
			capsuleHeight = 0.7,
			capsuleRadius = 0.25,
			collisionRadius = 0.6,			
			health = 300,
			protection = 15,
			evasion = 0,
			immunities = { "sleep", "blinded", "frozen" },
			resistances = {
				fire = "weak",
				shock = "weak",
				frost = "immune",
				poison = "immune",
			},
			traits = { "undead" },
			exp = 450,
			lootDrop = { 100, "long_sword", 100, "long_sword" },
		},
		{
			class = "SkeletonCommanderBrain",
			name = "brain",
			sight = 6,
			allAroundSight = true,
			morale = 100,	-- fearless
			onThink = function(self) 
				if (party.level == self.go.level) and math.abs(party.x - self.go.x) == 1 and math.abs(party.y - self.go.y) == 1 and
					math.abs(party.x - xan_boss_garth.x) <= 1 and math.abs(party.y - xan_boss_garth.y) <= 1 then
					self:performAction("summonUndead");
				elseif (party.level ~= self.go.level) or math.abs(party.x - xan_boss_garth.x) > 2 or math.abs(party.y - xan_boss_garth.y) > 1 then
					self:wait();
				else
					return false;
				end
			end,
		},
		{
			class = "MonsterMove",
			name = "move",
			sound = "skeleton_commander_walk",
			cooldown = 2,
		},
		{
			class = "MonsterTurn",
			name = "turn",
			sound = "skeleton_commander_walk",
		},
		{
			class = "MonsterAttack",
			name = "basicAttack",
			attackPower = 30,
			accuracy = 30,
			pierce = 20,
			woundChance = 25,
			cooldown = 3,
			sound = "skeleton_commander_attack",
			onBeginAction = function(self)
				-- randomize animation
				if math.random() < 0.5 then
					self:setAnimation("attack")
				else
					self:setAnimation("attack2")
				end
			end,
		},		
		{
			class = "MonsterAttack",
			name = "turnAttack",
			attackPower = 30,
			accuracy = 30,
			pierce = 20,
			woundChance = 25,
			cooldown = 0,
			sound = "skeleton_commander_turn_attack",
			turnToAttackDirection = true,
		},
		{
			class = "MonsterMoveAttack",
			name = "moveAttack",
			attackPower = 30,
			accuracy = 50,
			pierce = 20,
			woundChance = 25,
			cooldown = 6,
			animation = "moveAttack",
			sound = "skeleton_commander_move_attack",
		},
		{
			class = "MonsterAttack",
			name = "rangedAttack",
			attackType = "projectile",
			attackPower = 30,
			cooldown = 15,
			animation = "rangedAttack",
			animationSpeed = 1.5,
			sound = "ice_guardian_attack",
			shootProjectile = "frostbolt",
		},
		{
			class = "MonsterAttack",
			name = "summonUndead",
			cooldown = 5,
			repeatChance = 100,
			animation = "summonUndead",
			sound = "skeleton_commander_summon_spell",
			onAttack = function(self)
				for ix = -1, 1 do
					for iy = 0, 1 do
						local x = xan_boss_garth.x + ix;
						local y = xan_boss_garth.y + iy;
					
						local spell = spawn("ice_shards", self.go.level, x, y, self.go.facing, self.go.elevation)
						spell.iceshards:setRange(1)
						spell.tiledamager:setAttackPower(32)
					end
				end
			end,
		},
		{
			class = "Particle",
			name = "handLeftParticle",
			parentNode = "hand_left",
			particleSystem = "skeleton_commander_spell",
			offset = vec(0.1, 0, 0),
			enabled = false,
		},
		{
			class = "Particle",
			name = "handRightParticle",
			parentNode = "hand_right",
			particleSystem = "skeleton_commander_spell",
			offset = vec(0.1, 0, 0),
			enabled = false,
		},
		{
			class = "UggardianFlames",
			name = "aliveParticles",
			particleSystem = "ice_golem_particles",
			enabled = false,
		},		
		{
			class = "Light",
			name = "spellLight",
			parentNode = "spine1",
			offset = vec(0.4, 0, 0.25),
			--debugDraw = true,
			color = vec(0.45,0.6,1),
			brightness = 8,
			range = 4,
			fadeOut = 0,
			fillLight = true,
		},
	},
}

defineAnimationEvent{
	animation = "mod_assets/rooms/xan/assets/guardian_cast_spell.fbx",
	event = "summon_undead_begin",
	frame = 80,
}

defineAnimationEvent{
	animation = "mod_assets/rooms/xan/assets/guardian_cast_spell.fbx",
	event = "summon_undead_end",
	frame = 155,
}

defineAnimationEvent{
	animation = "mod_assets/rooms/xan/assets/guardian_cast_spell.fbx",
	event = "attack",
	frame = 155,
}





















