-- THIS CONTAINS ALL HOOKS USED IN XANATHAR'S ROOM AND DISPATCHES TO OTHER SCRIPTS

g_bootsEquipped = false;
g_restMessage = nil;
g_restUneasy = false;
g_restUneasyCount = 0;
g_restTime = 0;
g_goSylinaEnding = false;
g_goEugeneEnding = false;


function _tableRandom(table)
	local idx = math.random(1, #table);
	return table[idx];
end

function _getXanRoomFloor(go)
	if (go.level == xan_corner_1A.level) then
		if (go.x >= xan_corner_1A.x) and (go.y >= xan_corner_1A.y) and (go.x <= xan_corner_1B.x) and (go.y <= xan_corner_1B.y) then 
			return "above";
		else
			return nil;
		end
	elseif (go.level == xan_corner_2A.level) then
		if (go.x >= xan_corner_2A.x) and (go.y >= xan_corner_2A.y) and (go.x <= xan_corner_2B.x) and (go.y <= xan_corner_2B.y) then 
			return "below";
		else
			return nil;
		end
	else
		return nil;
	end
end


function _find(name)
	for l = 1, Dungeon.getMaxLevels() do
		for e in Dungeon.getMap(l):allEntities() do
			if (e.name == name) then
				return e;
			end
		end
	end
	
	return nil;
end


function onRest(self)
	local xanRoomFloor = _getXanRoomFloor(party);
	local endingState = xan_ending.script.getEndingState();
	local message = nil;
	local uneasy = false;
	
	local comeGetMeMessage = _tableRandom {
				"Come.. get me.. we will be powerful together..",
				"What are you feeling ? Fear ? I'm here to protect you. Come get me.",
				"Freedom. That is what you want. And that is what I want. Let's get it together.",
			};

	if (endingState == "not_found") then
		if (xanRoomFloor ~= nil) then
			message = comeGetMeMessage;
		end
	end	
	
	if (endingState == "maintalk") then
		if (xanRoomFloor ~= nil) then
			message = "Stay faithful to me.. We will be great together.";
		end
	end	

	if (endingState == "waiting_gem") then
		if (not party.party:isCarrying("xan_chaos_soulgem")) then
			if (xanRoomFloor) then
				message = comeGetMeMessage;
			else
				local soulgem = _find("xan_chaos_soulgem");
			
				if (soulgem) then
					if (_getXanRoomFloor(soulgem)) then
						message = comeGetMeMessage;
					else
						message = "You. You abandoned me. I won't be yours anymore.";
						g_goEugeneEnding = true;
					end
				end
			end 
		elseif (xanRoomFloor == "below") then
			message = _tableRandom {
				"Stay faithful to me.. We will be great together.",
				"Don't believe what they said. They don't want you to get as powerful as they once were.",
			};
		elseif (xanRoomFloor == "above") then
			message = "Let's leave this place. Quickly..";
		else
			message = _tableRandom {
				"Let's leave this place. The Red Hills are waiting for us!",
				"There's a place in the Red Hills where we can find what we are looking for.",
				"Feeling bored ? I know a place in Red Hill Town where we can party, party, party!",
				"Let's go take that treasure in the Red Hills, we'll never work any more!",
			};
			uneasy = true;
		end
	end	
	
	if (message ~= nil) then
		g_restMessage = message;
		g_restUneasy = uneasy;
		
		if (uneasy) then
			g_restUneasyCount = g_restUneasyCount + 1;
		end
		
		g_restTime = Time.currentTime();
		
		if (endingState == "waiting_gem") and (g_restUneasy) and (g_restUneasyCount >= 3) then
			g_goSylinaEnding = true;
		end
	end
end



function onWakeUp(self)
	if (g_goSylinaEnding) then
		xan_ending.script.sylinaEnding();
		hudPrint "You wake up, noticing something strange has happened.";
		g_goSylinaEnding = false;
	elseif (g_goEugeneEnding) then
		local soulgem = _find("xan_chaos_soulgem");
		xan_ending.script.abandonedEnding(soulgem);	
		g_goEugeneEnding = false;
	elseif (g_restUneasy) then
		g_restUneasy = false;
		hudPrint "These dreams are starting to make you uneasy. There is disagreement in the party on whether to keep the soulgem or not.";
	end

	g_restMessage = nil;
end


function onDrawGui(self, context)

	-- Dispatch cheats
	if (Editor.isRunning() and context.keyDown("X") and context.keyDown(string.char(17))) then
		xan_cheat.script.showSelectMenu();
	elseif (Editor.isRunning() and context.keyDown("K") and context.keyDown(string.char(17))) then
		xan_cheat.script.armageddon();
	end

	-- Rest animation
	if (g_restMessage ~= nil) then
		local time = Time.currentTime() - g_restTime;
		
		time = time - 4.5;
		
		if (time < 0) then
			return;
		end
	
		local alpha0 = math.min(255, math.max(0, 255 * time));
		local alpha1 = math.min(255, math.max(0, 64 * (time - 1.0)));
	
		context.color(0, 0, 0, alpha0);
		context.drawRect(0, 0, context.width, context.height);

		context.color(255, 255, 255, alpha1);
		
		
		local W = context.width;
		local H = context.height;
		
		x = (W - 1024) / 2;
		y = (H - 512) / 4;
		
		context.drawImage("mod_assets/rooms/xan/assets/sshot2x1ink.dds", x, y); 
		
		local substridx = math.min(#g_restMessage, math.max(1, (time - 5.0) * 2.0));
		local str = g_restMessage:sub(1, substridx);
		
		context.color(255, 128, 255, alpha1);

		context.font("large");
		
		context.drawParagraph(str, x, y + 552, 1024);
	end

	-- Blinding boots
	if (g_bootsEquipped) then
		context.color(0, 0, 0, 240);
		context.drawRect(0, 0, context.width, context.height);
	end
	
	-- Soulgem UI
	xan_wildmage.script.drawUI(context);
end


function blindingBootsEquip(self, champion, slot, equipped)
	if (slot ~= ItemSlot.Feet) then return end

	if (equipped) then
		party.party:setMovementSpeed(2.5);
		g_bootsEquipped = true;
	else
		g_bootsEquipped = false;
		party.party:setMovementSpeed(1.0);
	end
end

function _isSoulgem(item)
	return (item ~= nil) and (item.go.name == "xan_chaos_soulgem");
end

function onCastSpell(self, champion, spell)
	local hand1 = champion:getItem(ItemSlot.Weapon)
	local hand2 = champion:getItem(ItemSlot.OffHand)

	if (_isSoulgem(hand1) or _isSoulgem(hand2)) then
		xan_wildmage.script.wildSurgeRandom(champion, 20);
	elseif (champion:hasTrait("xan_wild_mage")) then
		xan_wildmage.script.wildSurgeRandom(champion, 5);
	end
end

party.party:addConnector("onDrawGui", self.go.id, "onDrawGui");
party.party:addConnector("onWakeUp", self.go.id, "onWakeUp");
party.party:addConnector("onRest", self.go.id, "onRest");
party.party:addConnector("onCastSpell", self.go.id, "onCastSpell");

