-- Script of all things in the tomb/dungeon level of Xan room

-- ========================================
-- INITIALIZATIONS
-- ========================================

xan_mimic_chest_1.chest:setMimic(true);
xan_timer_golem_attack.controller:stop();
xan_spike_timer.controller:stop();
xan_timer_ending.controller:stop();

-- ========================================
-- BOSS FIGHTS
-- ========================================

function startOtisFight()
	xan_boss_otis.bossfight:addMonster(xan_otis.monster);
	xan_boss_otis.bossfight:activate();
end

function startGarthFight()
	xan_boss_garth.bossfight:addMonster(xan_garth.monster);
	xan_boss_garth.bossfight:activate();
end

-- ========================================
-- SPIKE TRAP
-- ========================================

g_SpikeTrapCounter = 0;

g_Spikes = {
	{ 0, 1, 1, 1, 1, 0},
	{ 0, 0, 1, 0, 0, 0},
	{ 1, 0, 1, 0, 1, 1},
	{ 0, 0, 0, 1, 1, 1},
	{ 1, 1, 0, 0, 1, 1},
	{ 1, 1, 1, 1, 0, 0},
}

function spikeTrap()

	g_SpikeTrapCounter = (g_SpikeTrapCounter + 1) % (#g_Spikes);
	
	local spikes = g_Spikes[g_SpikeTrapCounter + 1];
	
	_setSpike(xan_spike_1, spikes[1]);
	_setSpike(xan_spike_2, spikes[2]);
	_setSpike(xan_spike_3, spikes[3]);
	_setSpike(xan_spike_4, spikes[4]);
	_setSpike(xan_spike_5, spikes[5]);
	_setSpike(xan_spike_6, spikes[6]);
end

function _setSpike(spike, digit)
	if (digit == 0) then 
		spike.controller:deactivate();
	else 
		spike.controller:activate();
	end
end

-- ========================================
-- GOLEM PUZZLE
-- ========================================

g_Golem1State = "enemy"; -- "enemy", "idle" or "friend"
g_Golem2State = "enemy";

function _chance(perc)
	return math.random(0, 99) < perc;
end

-- returns distance values between two entities - maxdist, dx, dy
-- dx+dy is taxicab distance, dx is distance over axis x, dy over axis y
function _dist(o1, o2)
	if (o1 == nil or o2 == nil) then
		return 999, 999, 999;
	end

	local dx = math.abs(o1.x - o2.x);
	local dy = math.abs(o1.y - o2.y) 
	return math.max(dx, dy), dx, dy;
end

-- returns the relative direction of target as viewed by src
-- works only if src is in the immediate tiles in front, left, right, behind or under target.
function _direction(src, target, dx, dy)
	if (src.x == target.x and src.y == target.y) then
		return "under";
	end

	local fx, fy = getForward(src.facing);

	if (src.x + fx == target.x) then
		if (src.y + fy == target.y) then
			return "front";
		else
			return "behind";
		end
	elseif (src.y + fy == target.y) then
		return "behind";
	end
	
	fx, fy = getForward((src.facing + 1) % 4);
	
	if (src.x + fx == target.x) then
		if (src.y + fy == target.y) then
			return "right";
		else
			return "left";
		end
	elseif (src.y + fy == target.y) then
		return "left";
	end
	
	return "???";
end


function _execGolemAttack(dist, dx, dy, golem)
	if (dist == 1) then
		if (dx >= 1 and dy >= 1) then
			golem.brain:performAction("groundPound");
		else 
			local dir = _direction(golem, party);
			
			if (dir == "left") then
				golem.brain:turnAttackLeft();
			elseif (dir == "right") then
				golem.brain:turnAttackRight();
			elseif (dir == "front") then
				golem.brain:performAction("basicAttack");
			elseif (dir == "behind") then
				golem.brain:turnAttackRight();
			else
				golem.brain:performAction("groundPound");
			end
		end
	else
		golem.brain:performAction("rangedAttack");
	end
end

function doGolemAttack()
	if (xan_magma_golem_1 == nil or xan_magma_golem_2 == nil) then return; end

	local d1, dx1, dy1 = _dist(party, xan_magma_golem_1);
	local d2, dx2, dy2 = _dist(party, xan_magma_golem_2);

	if (d1 <= 1 or d2 <= 1) then		
		-- in pit
		if (g_Golem1State == "enemy") then
			_execGolemAttack(d1, dx1, dy1, xan_magma_golem_1);
		end
		if (g_Golem2State == "enemy") then
			_execGolemAttack(d2, dx2, dy2, xan_magma_golem_2);
		end
	else
		-- outside pit
		xan_timer_golem_attack.controller:stop();
	end
end


function _checkDisableGolem(altar, jammerName, forceField, golem)
	local found = false;
	local count = 0;
	
	for v, i in altar.surface:contents() do
		if (i.go.name == jammerName) then
			found = true;
		end
		count = count + 1;
	end
	
	if (found and (count == 1)) then
		forceField.controller:activate();
		golem.aliveParticles:stop();
		playSound "spell_expires";
		hudPrint "The golem life force suddenly disappears, as the flame of a snuffed candle.";
		return "idle";
	elseif found then
		hudPrint "The presence of other objects on the altar is preventing its power to work.";
	end
	
	return "enemy";
end


function checkDisableGolem1()
	if (xan_magma_golem_1 == nil or xan_magma_golem_2 == nil) then return; end

	if (g_Golem1State == "enemy") then
		g_Golem1State = _checkDisableGolem(xan_golem_altar_1, "xan_fire_jammer", xan_golem_force_field_1, xan_magma_golem_1);
	end
end

function checkDisableGolem2()
	if (xan_magma_golem_1 == nil or xan_magma_golem_2 == nil) then return; end

	if (g_Golem2State == "enemy") then
		g_Golem2State = _checkDisableGolem(xan_golem_altar_2, "xan_ice_jammer", xan_golem_force_field_2, xan_magma_golem_2);
	end
end

g_String = "";
SOLUTION = "14125";

function _checkEndSequence(chr)
	g_String = g_String .. chr;
	
	if (#g_String > #SOLUTION) then
		g_String = g_String:sub(2);
	end
	
	-- print(g_String);
	
	if (g_String == SOLUTION) then
		xan_fake_wall_2.door:open();
		xan_endb_1.controller:activate();
		xan_endb_2.controller:activate();
		xan_endb_3.controller:activate();
		playSound "light";
	end
end


function btn1()
	_checkEndSequence "1"
end

function btn2()
	_checkEndSequence "2"
end

function btn3()
	_checkEndSequence "3"
end

function btn4()
	_checkEndSequence "4"
end

function btn5()
	_checkEndSequence "5"
end






